/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Copyright (C) 2010 France Telecom
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: MonitoringService.java 7487 2011-01-21 16:04:41Z durieuxp $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.monitoring.eos.monitoring.service;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.Timer;
import java.util.TimerTask;

import javax.management.MalformedObjectNameException;
import javax.management.ObjectName;

import org.ow2.jasmine.event.beans.JasmineEventEB;
import org.ow2.jasmine.monitoring.eos.common.JasmineEventManager;
import org.ow2.jasmine.monitoring.eos.common.jms.JMSConnection;
import org.ow2.jasmine.monitoring.eos.common.jms.event.JasmineMessageEvent;
import org.ow2.jasmine.monitoring.eos.monitoring.ConfigFile;
import org.ow2.jasmine.monitoring.eos.monitoring.DataBaseEventExtractor;
import org.ow2.jasmine.monitoring.eos.monitoring.EventData;
import org.ow2.jasmine.monitoring.eos.monitoring.ReplayFileName;
import org.ow2.jasmine.monitoring.eos.monitoring.Serie;
import org.ow2.jasmine.monitoring.eos.monitoring.SerieResult;
import org.ow2.jonas.lib.bootstrap.JProp;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;

/**
 * Services provided by the monitoring module.
 *
 * @author Guillaume Renault
 *
 */
public class MonitoringService implements Serializable {

    /**
     * serialVersionUID
     */
    private static final long serialVersionUID = 8402643661543808499L;

    /**
     * Logger.
     */
    private Log logger = LogFactory.getLog(this.getClass());

    private Set<String> replaydirlist;

    private Set<String> configdirlist;

    /**
     * Timer for disconnecting the JMS connection.
     */
    private transient Timer timer = null;

    /**
     * Time at which to disconnect the JMS topic connection.
     */
    private long disconnectTime = 0;

    /**
     * JMS check interval.
     */
    private static final long JMS_CHECK_INTERVAL = 1000;

    /**
     * JMS timeout.
     */
    private static final long JMS_TIMEOUT = 60 * 1000;

    /**
     * JMS topic used for synchronization.
     */
    private static final String JMS_TOPIC = "jasmine";

    private JMSConnection jmsConnection = null;

    private JasmineEventManager eventManager = null;

    public MonitoringService() {
        readConfig();
        this.eventManager = new JasmineEventManager();
    }

    /**
     * Test if the file is a Config file
     * @return
     */
    private boolean isConfigFile(final File file) {
        FileReader fileReader = null;
        BufferedReader buffer = null;
        try {
            fileReader = new FileReader(file);
            buffer = new BufferedReader(fileReader);
            String line = "";
            while ((line = buffer.readLine()) != null) {
                if (line.contains("<monitor")) {
                    return true;
                }
            }
        } catch (Exception e) {
        } finally {
            if (buffer != null) {
                try {
                    buffer.close();
                } catch (Exception e) {
                    logger.error("Failed closing buffer", e);
                }
            }
            if (fileReader != null) {
                try {
                    fileReader.close();
                } catch (Exception e) {
                    logger.error("Failed closing fileReader", e);
                }
            }
        }
        return false;
    }

    /**
     * Load an xml file to configure graphs.
     *
     * @param path the xml file to load
     * @return a string representing the xml file, without the header.
     */
    public String loadConfigFile(final String path) throws Exception {
        logger.debug("Load config file: " + path);
        String result = "";
        FileReader fileReader = null;
        BufferedReader buffer = null;
        File file = new File(path);
        if (! file.exists()) {
            file = new File(JProp.getJonasBase() + File.separator + path);
        }
        if (! file.exists()) {
            throw new Exception("Impossible to locate the file: " + path);
        }
        String endofline = System.getProperty("line.separator");
        try {
            fileReader = new FileReader(file);
            buffer = new BufferedReader(fileReader);
            boolean write = false;
            String line = "";
            while ((line = buffer.readLine()) != null) {
                if (line.contains("<monitor")) {
                    write = true;
                }
                if (write) {
                    result += line + endofline;
                }
            }
            if (! write) {
                throw new Exception("Bad configuration file: No 'monitor' tag found.");
            }
            return result;
        } finally {
            if (buffer != null) {
                try {
                    buffer.close();
                } catch (Exception e) {
                    logger.error("Failed closing buffer", e);
                }
            }
            if (fileReader != null) {
                try {
                    fileReader.close();
                } catch (Exception e) {
                    logger.error("Failed closing fileReader", e);
                }
            }
        }
    }

    /**
     * Load data from a log file.
     * 2 formats are possibles : header (-f option) or no header (eventswitch)
     * @param fileName
     * @param series
     * @return
     * @throws Exception
     */
    public List<SerieResult> loadLogFile(final String fileName, final List<Serie> series) throws Exception {
        logger.debug("Start reading file: " + fileName);
        FileReader f = new FileReader(fileName);
        BufferedReader buffer = new BufferedReader(f);
        String header = buffer.readLine();
        if (header == null) {
            logger.warn("Empty file: " + fileName);
            throw new Exception("Empty file: " + fileName);
        }
        if (header.startsWith("date")) {
            return loadLogFileHeader(buffer, series, header, 0);
        } else if (header.startsWith("time")) {
            return loadLogFileHeader(buffer, series, header, 1);
        } else {
            // we lost the first line... not a big deal!
            return loadLogFileNoHeader(buffer, series);
        }
    }

    /**
     * Load data from a log file with no header
     * The format of each line is decided by the EventSwitch:
     * time;date;sname;domain;server;mbean;value
     *    time   = time in millisec
     *    date   = 2010/12/22 00:00:00
     *    sname  = server URL : service:jmx:rmi:///jndi/...
     *    domain = domain name
     *    server = server name (last part of sname usually)
     *    mbean  = <mbean name>:<attribute>
     *    value  = attribute value
     * @param buffer
     * @param series
     * @return
     * @throws Exception
     */
    private List<SerieResult> loadLogFileNoHeader(final BufferedReader buffer, final List<Serie> series) throws Exception {

        // each Serie has its own list of results
        Map<String, ArrayList<EventData>> datalist = new HashMap<String, ArrayList<EventData>>();
        for (Serie serie : series) {
            datalist.put(serie.getId(), new ArrayList<EventData>());
        }

        // parse the file
        String line = "";
        while ((line = buffer.readLine()) != null) {
            //logger.debug("Reading line:" + line);
            // split the line just once
            String[] split = line.split(";");
            if (split.length <= 6) {
                // Ignore all lines incomplete.
                // This may happen in case of "poll -slb" for example.
                //logger.debug("line incomplete");
                continue;
            }
            String date = split[1];
            String server = split[4];

            // compute mbean and attribute
            String mbeanattr = split[5];
            int index = mbeanattr.lastIndexOf(":");
            String mbean = mbeanattr.substring(0, index);
            String attr = mbeanattr.substring(index+1);

            String value = split[6];

            // Find series accepting this event.
            for (Serie curser : series) {
                // Check Attribute
                if (! attr.equals(curser.getYcol())) {
                    //logger.debug("bad attr:" + curser.getYcol());
                    continue;
                }

                // Check pattern
                if (! match(mbean, curser.getPattern())) {
                    continue;
                }

                // Check server
                if (curser.getServer() != null && curser.getServer().length() > 0 && ! curser.getServer().equals(server)) {
                    //logger.debug("bad server:" + curser.getServer());
                    continue;
                }

                // This event matches this Serie and will be kept
                EventData ed = new EventData(date, value);
                datalist.get(curser.getId()).add(ed);
                // continue because an event may match several series
            }
        }

        // return list of results
        List<SerieResult> result = new ArrayList<SerieResult>();
        for (Serie ser : series) {
            SerieResult sr = new SerieResult();
            sr.setId(ser.getId());
            sr.setResults(datalist.get(sr.getId()));
            result.add(sr);
            logger.debug("Found " + sr.getResults().size() + " events for serie " + sr.getId());
        }
        return result;
    }

    /**
     * Version when header in file
     * TODO This will be deprecated when -f uses the same format than eventswitch:
     * - no header
     * - 1 line by attribute
     * @param buffer
     * @param series
     * @param header complete header (first line)
     * @param dateind index date in the header (0 or 1)
     * @return
     * @throws Exception
     */
    private List<SerieResult> loadLogFileHeader(final BufferedReader buffer, final List<Serie> series, final String header, final int dateind) throws Exception {

        // Parse the header to look for column names
        String[] headerArray = header.split(";");
        int mbeanCol = -1;
        int serverCol = -1;
        Map<String, Integer> valueCol = new HashMap<String, Integer>();
        for (int index = 0; index < headerArray.length; index++) {
            if (headerArray[index].equals("server")) {
                serverCol = index;
            }
            if (headerArray[index].equals("mbean")) {
                mbeanCol = index;
            }
            for (Serie afil : series) {
                if (afil.getYcol().equals(headerArray[index])) {
                    valueCol.put(afil.getId(), new Integer(index));
                }
            }
        }
        if (serverCol == -1) {
            throw new Exception("Bad log file: No server in header");
        }
        if (mbeanCol == -1) {
            throw new Exception("Bad log file: No mbean in header");
        }

        // each Serie has its own list of results
        Map<String, ArrayList<EventData>> datalist = new HashMap<String, ArrayList<EventData>>();
        for (Serie serie : series) {
            datalist.put(serie.getId(), new ArrayList<EventData>());
        }

        // parse the file
        String line = "";
        while ((line = buffer.readLine()) != null) {
            // split the line just once
            String[] split = line.split(";");
            if (split.length < headerArray.length) {
                // Ignore all lines incomplete.
                // This may happen in case of "poll -slb" for example.
                continue;
            }
            String date = split[dateind];
            String server = split[serverCol];

            // compute mbean
            String mbeanattr = split[mbeanCol];
            int index = mbeanattr.lastIndexOf(":");
            String mbean = mbeanattr.substring(0, index);

            // Find series accepting this event.
            for (Serie curser : series) {
                Integer valueIndex = valueCol.get(curser.getId());
                if (valueIndex == null) {
                    // This serie is not present in this file.
                    continue;
                }
                String value = split[valueIndex.intValue()];

                // Check pattern
                if (! match(mbean, curser.getPattern())) {
                    continue;
                }

                // Check server
                if (curser.getServer() != null && curser.getServer().length() > 0 && ! curser.getServer().equals(server)) {
                    continue;
                }

                // This event matches this Serie and will be kept
                EventData ed = new EventData(date, value);
                datalist.get(curser.getId()).add(ed);
                // continue because an event may match several series
            }
        }

        // return list of results
        List<SerieResult> result = new ArrayList<SerieResult>();
        for (Serie ser : series) {
            SerieResult sr = new SerieResult();
            sr.setId(ser.getId());
            sr.setResults(datalist.get(sr.getId()));
            result.add(sr);
            logger.debug("Found " + sr.getResults().size() + " events for serie " + sr.getId());
        }
        logger.debug("Stop reading file:" + result.size());
        return result;
    }

    /**
     * Loading data from DataBase
     *
     * @param args the arguments to access to database
     * @param filter the fields to get.
     * @return
     * @throws Exception
     */
    public List<SerieResult> loadFromDataBase(final Date from, final Date to, final List<Serie> series) throws Exception {

        logger.debug("start reading database");

        // each Serie has its own list of results
        Map<String, ArrayList<EventData>> datalist = new HashMap<String, ArrayList<EventData>>();
        for (Serie serie : series) {
            datalist.put(serie.getId(), new ArrayList<EventData>());
        }

        // Use a Set to avoid duplicate elements
        Set<String> servers = new HashSet<String>();
        Set<String> probes = new HashSet<String>();
        for (Serie element : series) {
            probes.add(element.getYcol());
            if (element.getServer() != null) {
                servers.add(element.getServer());
            }
        }

        // Connect to EJB and get the events from database
        DataBaseEventExtractor dataBaseEventExtractor = new DataBaseEventExtractor();
        JasmineEventEB[] events = dataBaseEventExtractor.getEvents(servers, probes, from, to);

        if (events != null) {

            // return only event matching one of the Series
            SimpleDateFormat format = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss" );
            for (JasmineEventEB elem : events) {

                // Find series accepting this event.
                for (Serie curser : series) {

                    // Check Attribute
                    if (! elem.getProbe().endsWith(":"+curser.getYcol())) {
                        continue;
                    }

                    // Check pattern
                    if (! match(elem.getMBean(), curser.getPattern())) {
                        continue;
                    }

                    // Check server
                    if (curser.getServer() != null && curser.getServer().length() > 0 && ! curser.getServer().equals(elem.getServer())) {
                        continue;
                    }

                    // This event matches this Serie and will be kept
                    String dateConverted = format.format(elem.getTimestamp());
                    EventData ed = new EventData(dateConverted, elem.getValue().toString());
                    datalist.get(curser.getId()).add(ed);
                    // continue because an event may match several series
                }
            }
        }

        // return list of results
        List<SerieResult> result = new ArrayList<SerieResult>();
        for (Serie ser : series) {
            SerieResult sr = new SerieResult();
            sr.setId(ser.getId());
            sr.setResults(datalist.get(sr.getId()));
            result.add(sr);
            logger.debug("Found " + sr.getResults().size() + " events for serie " + sr.getId());
        }
        logger.debug("Stop reading database:" + result.size());
        return result;
    }

    /**
     * Start live monitoring.
     *
     * @param series list of series we are interested to get
     * @return
     * @throws Exception
     */
    public List<SerieResult> startMonitoring(final List<Serie> series) throws Exception {

        // each Serie has its own list of results
        Map<String, ArrayList<EventData>> datalist = new HashMap<String, ArrayList<EventData>>();
        for (Serie serie : series) {
            datalist.put(serie.getId(), new ArrayList<EventData>());
        }

        // get all events not read yet.
        this.connectTopic();
        List<JasmineMessageEvent> events = this.eventManager.getAll();

        // return only event matching one of the Series
        for (JasmineMessageEvent elem : events) {

            // Find series accepting this event.
            for (Serie curser : series) {

                // Check Attribute
                if (! elem.getProbe().endsWith(":"+curser.getYcol())) {
                    //logger.debug("Attribute do not match :" + curser.getId());
                    continue;
                }

                // Check if pattern matches the mbean
                if (! match(elem.getMBean(), curser.getPattern())) {
                    //logger.debug("Mbean do not match :" + curser.getId());
                    continue;
                }

                // Check server
                if (curser.getServer() != null && curser.getServer().length() > 0 && ! curser.getServer().equals(elem.getServer())) {
                    //logger.debug("Server do not match :" + curser.getId());
                    continue;
                }

                // This event matches this Serie and will be kept
                EventData ed = new EventData(elem.getFormattedTimeStamp(), elem.getValue());
                datalist.get(curser.getId()).add(ed);
                // continue because an event may match several series
            }
        }

        // return list of results
        List<SerieResult> result = new ArrayList<SerieResult>();
        for (Serie ser : series) {
            SerieResult sr = new SerieResult();
            sr.setId(ser.getId());
            sr.setResults(datalist.get(sr.getId()));
            result.add(sr);
            logger.debug("Found " + sr.getResults().size() + " events for serie " + sr.getId());
        }
        return result;
    }

    /**
     * Stop live monitoring.
     * @throws Exception
     */
    public void stopMonitoring() throws Exception {
        this.disconnectTopic();
    }

    /**
     * Get the list of result files candidate to replay
     * @param filter used to filter the file names in directories
     * @return the list of files
     * @throws Exception If directory does not exist
     */
    public ArrayList<ReplayFileName> getReplayFilenames(final String filter) throws Exception {
        logger.debug(filter);

        ArrayList<ReplayFileName> result = new ArrayList<ReplayFileName>();
        for (String dir : replaydirlist) {
            result.addAll(getReplayFiles(dir, filter));
        }
        return result;
    }

    private ArrayList<ReplayFileName> getReplayFiles(final String directory, final String filter) {
        logger.debug(directory);
        ArrayList<ReplayFileName> result = new ArrayList<ReplayFileName>();

        // Try path as absolute, then as relative to JONAS_BASE.
        File dir = new File(directory);
        if (! dir.exists()) {
            dir = new File(JProp.getJonasBase() + File.separator + directory);
        }
        if (! dir.exists()) {
            logger.error("cannot find " + directory);
            return result;
        }
        File[] files = dir.listFiles(new NameFilter(filter));
        if (files == null) {
            logger.warn("no files found in " + directory);
            return result;
        }
        for (int i = 0; i < files.length; i++) {
            ReplayFileName rfn = new ReplayFileName();
            rfn.setDirectory(dir.getPath());
            rfn.setName(files[i].getName());
            rfn.setSize(files[i].length());
            rfn.setDate(new Date(files[i].lastModified()));
            rfn.setSeparator(File.separator);
            result.add(rfn);
            logger.debug("found " + files[i].getName());
        }
        return result;
    }

    /**
     * Save an XML config file
     * @param name the name of the file
     * @param file the XML data to save
     * @throws Exception
     */
    public ConfigFile saveConfigFile(final String name, final String file) throws Exception {
        logger.error("Save config file:" + name);

        // Assume the file will be put in the "monitoring" directory
        File pathname = new File(JProp.getJonasBase() + File.separator + "monitoring" + File.separator + name);
        FileWriter f = new FileWriter(pathname);
        f.write(file);
        f.close();

        // return the ConfigFile VO to be added to the list
        ConfigFile rfn = new ConfigFile();
        rfn.setDirectory("monitoring");
        rfn.setName(name);
        rfn.setSize(pathname.length());
        rfn.setDate(new Date(pathname.lastModified()));
        rfn.setSeparator(File.separator);
        return rfn;
    }

    public ArrayList<ConfigFile> removeConfigFile(final ConfigFile file) throws Exception {
        logger.error("Remove config file:" + file.getName());

        // Remove the file
        for (String strdir : configdirlist) {
            if (strdir.endsWith(file.getDirectory())) {
                // Try path as absolute, then as relative to JONAS_BASE.
                File dir = new File(strdir);
                if (! dir.exists()) {
                    dir = new File(JProp.getJonasBase() + File.separator + strdir);
                }
                if (! dir.exists()) {
                    logger.error("cannot find " + strdir);
                    break;
                }
                File f = new File(dir, file.getName());
                f.delete();
                break;
            }
        }

        // return the new list.
        // Filter used to build the file list
        // TODO Use the same filter as for getConfigFiles
        String filter = "*.xml";
        ArrayList<ConfigFile> result = new ArrayList<ConfigFile>();
        for (String dir : configdirlist) {
            result.addAll(getConfigFiles(dir, filter));
        }
        return result;
    }

    public ArrayList<ConfigFile> renameConfigFile(final ConfigFile file, final String newname) throws Exception {
        logger.error("Rename config file:" + file.getName());

        // Rename the file
        for (String strdir : configdirlist) {
            if (strdir.endsWith(file.getDirectory())) {
                // Try path as absolute, then as relative to JONAS_BASE.
                File dir = new File(strdir);
                if (! dir.exists()) {
                    dir = new File(JProp.getJonasBase() + File.separator + strdir);
                }
                if (! dir.exists()) {
                    logger.error("cannot find " + strdir);
                    break;
                }
                File f = new File(dir, file.getName());
                File dest = new File(dir, newname);
                f.renameTo(dest);
                break;
            }
        }

        // return the new list.
        // Filter used to build the file list
        // TODO Use the same filter as for getConfigFiles
        String filter = "*.xml";
        ArrayList<ConfigFile> result = new ArrayList<ConfigFile>();
        for (String dir : configdirlist) {
            result.addAll(getConfigFiles(dir, filter));
        }
        return result;
    }

    /**
     * Get the list of configuration files
     * @param filter used to filter the file names in directories
     * @return the list of files
     * @throws Exception If directory does not exist
     */
    public ArrayList<ConfigFile> getConfigFiles(final String filter) throws Exception {
        logger.debug(filter);

        ArrayList<ConfigFile> result = new ArrayList<ConfigFile>();
        for (String dir : configdirlist) {
            result.addAll(getConfigFiles(dir, filter));
        }
        return result;
    }

    // --------------------------------------------------------------------------
    // Private functions
    // --------------------------------------------------------------------------

    private boolean match(final String strmbean, final String strpattern) {
        if (strpattern == null || strpattern.length() == 0 || strpattern.equals("*:*")) {
            // No pattern defined.
            return true;
        }
        //logger.debug("compare pattern:" + strpattern + " to mbean:" + strmbean);
        ObjectName mbean = null;
        ObjectName pattern = null;
        try {
            mbean = new ObjectName(strmbean);
        } catch (MalformedObjectNameException e) {
            logger.error("Bad MBean ObjectName (MalformedObjectNameException) " + strmbean);
            return false;
        } catch (NullPointerException e) {
            logger.error("Bad MBean ObjectName (NPE) " + strmbean);
            return false;
        }
        try {
            pattern = new ObjectName(strpattern);
        } catch (MalformedObjectNameException e) {
            logger.error("Bad Pattern ObjectName (MalformedObjectNameException) " + strpattern);
            return false;
        } catch (NullPointerException e) {
            logger.error("Bad Pattern ObjectName (NPE) " + strpattern);
            return false;
        }
        if (pattern == null) {
            logger.error("Cannot get ObjectName on " + strpattern);
            return false;
        }

        return pattern.apply(mbean);
    }

    private ArrayList<ConfigFile> getConfigFiles(final String directory, final String filter) {
        ArrayList<ConfigFile> result = new ArrayList<ConfigFile>();

        // Try path as absolute, then as relative to JONAS_BASE.
        File dir = new File(directory);
        if (! dir.exists()) {
            dir = new File(JProp.getJonasBase() + File.separator + directory);
        }
        if (! dir.exists()) {
            logger.error("cannot find " + directory);
            return result;
        }
        logger.debug("directory explored=" + dir.getPath());
        File[] files = dir.listFiles(new NameFilter(filter));
        if (files == null) {
            logger.warn("no files found in " + directory);
            return result;
        }
        for (int i = 0; i < files.length; i++) {
            if (isConfigFile(files[i])) {
                ConfigFile rfn = new ConfigFile();
                rfn.setDirectory(dir.getPath());
                rfn.setName(files[i].getName());
                rfn.setSize(files[i].length());
                rfn.setDate(new Date(files[i].lastModified()));
                rfn.setSeparator(File.separator);
                result.add(rfn);
                logger.debug("found " + files[i].getName());
            }
        }
        return result;
    }

    /**
     * read jasmine-monitoring.properties
     * the file may not exist.
     */
    private void readConfig() {
        JProp config = JProp.getInstance("jasmine-monitoring.properties");

        // directories for replay (usually .csv)
        replaydirlist = new HashSet<String>();
        String [] list = config.getValueAsArray("jasmine.monitoring.replaydir");
        if (list == null) {
            // Default is monitoring
            logger.debug("Missing jasmine-monitoring.properties: take default values");
            replaydirlist.add("monitoring");
        } else {
            for (int i = 0; i < list.length; i++) {
                replaydirlist.add(list[i]);
            }
        }
        // Always add the directory used for upload
        String eostemp = System.getProperty("java.io.tmpdir") + File.separatorChar + "EoS.temp";
        replaydirlist.add(eostemp);

        // directories for configuration files (.xml)
        configdirlist = new HashSet<String>();
        list = config.getValueAsArray("jasmine.monitoring.configdir");
        if (list == null) {
            // Default is monitoring
            logger.debug("Missing jasmine-monitoring.properties: take default values");
            configdirlist.add("monitoring");
        } else {
            for (int i = 0; i < list.length; i++) {
                configdirlist.add(list[i]);
            }
        }
        // Always add the directory used for upload
        configdirlist.add(eostemp);
    }

    /**
     * Connect to the JMS topic.
     */
    private void connectTopic() throws Exception {

        synchronized (this) {
            if (this.jmsConnection == null) {
                logger.debug("Connecting to the JMS topic...");
                this.jmsConnection = new JMSConnection(this.eventManager);
                this.jmsConnection.connect(MonitoringService.JMS_TOPIC);
            }
            //this.resetTimer();
        }
        logger.debug("Connected to the JMS topic");
    }

    /**
     * Disconnect from the JMS topic.
     */
    private void disconnectTopic() throws Exception {

        synchronized (this) {
            if (this.jmsConnection != null) {
                logger.debug("Disconnecting from the JMS topic...");
                this.jmsConnection.disconnect();
                this.jmsConnection = null;
            }

            this.disconnectTime = 0;

            if (this.timer != null) {
                this.timer.cancel();
                this.timer.purge();
                this.timer = null;
            }
        }
        logger.debug("Disconnected from the JMS topic");
    }

    /**
     * Reset the timer.
     */
    private void resetTimer() {
        synchronized (this) {
            this.disconnectTime = System.currentTimeMillis() + MonitoringService.JMS_TIMEOUT;

            if (this.timer == null) {
                this.timer = new Timer();
                this.timer.schedule(new DisconnectTask(), MonitoringService.JMS_CHECK_INTERVAL,
                    MonitoringService.JMS_CHECK_INTERVAL);
            }
        }
    }

    /**
     * Implementation of TimerTask.
     *
     * @author Guillaume Renault
     */
    class DisconnectTask extends TimerTask {

        @Override
        public void run() {
            try {
                if (System.currentTimeMillis() > disconnectTime) {
                    disconnectTopic();
                }
            } catch (Throwable t) {
                logger.error("Failed disconnecting from JMS topic", t);
            }
        }
    }

    @SuppressWarnings("unused")
    private void readObject(final ObjectInputStream in) throws IOException, ClassNotFoundException {
        // deserialize everything except for the transient fields
        in.defaultReadObject();

        if (this.disconnectTime > 0) {
            // relaunch JMS timer
            this.resetTimer();
        }
    }

}
